@@ -4,10 +4,9 @@
 use crate::{
     cluster_info_vote_listener::VerifiedVoteReceiver,
     cluster_slots::ClusterSlots,
-    cluster_slots_service::ClusterSlotsService,
+    cluster_slots_service::{ClusterSlotsService, ClusterSlotsUpdateReceiver},
     completed_data_sets_service::CompletedDataSetsSender,
-    repair_service::DuplicateSlotsResetSender,
-    repair_service::RepairInfo,
+    repair_service::{DuplicateSlotsResetSender, RepairInfo},
     result::{Error, Result},
     window_service::{should_retransmit_and_persist, WindowService},
 };
@@ -429,7 +428,9 @@ fn retransmit(
         // neighborhood), then we expect that the packet arrives at tvu socket
         // as opposed to tvu-forwards. If this is not the case, then the
         // turbine broadcast/retransmit tree is mismatched across nodes.
-        if packet.meta.forward == (my_index % DATA_PLANE_FANOUT == 0) {
+        let anchor_node = my_index % DATA_PLANE_FANOUT == 0;
+        if packet.meta.forward == anchor_node {
+            // TODO: Consider forwarding the packet to the root node here.
             retransmit_tree_mismatch += 1;
         }
         peers_len = cmp::max(peers_len, shuffled_stakes_and_index.len());
@@ -465,10 +466,19 @@ fn retransmit(
             .or_default() += 1;
 
         let mut retransmit_time = Measure::start("retransmit_to");
-        if !packet.meta.forward {
-            ClusterInfo::retransmit_to(&neighbors, packet, sock, true)?;
+        // If the node is on the critical path (i.e. the first node in each
+        // neighborhood), it should send the packet to tvu socket of its
+        // children and also tvu_forward socket of its neighbors. Otherwise it
+        // should only forward to tvu_forward socket of its children.
+        if anchor_node {
+            ClusterInfo::retransmit_to(&neighbors, packet, sock, /*forward socket=*/ true);
         }
-        ClusterInfo::retransmit_to(&children, packet, sock, packet.meta.forward)?;
+        ClusterInfo::retransmit_to(
+            &children,
+            packet,
+            sock,
+            !anchor_node, // send to forward socket!
+        );
         retransmit_time.stop();
         retransmit_total += retransmit_time.as_us();
     }
@@ -559,8 +569,8 @@ pub fn retransmitter(
                             &rpc_subscriptions,
                         ) {
                             match e {
-                                Error::RecvTimeoutError(RecvTimeoutError::Disconnected) => break,
-                                Error::RecvTimeoutError(RecvTimeoutError::Timeout) => (),
+                                Error::RecvTimeout(RecvTimeoutError::Disconnected) => break,
+                                Error::RecvTimeout(RecvTimeoutError::Timeout) => (),
                                 _ => {
                                     inc_new_counter_error!("streamer-retransmit-error", 1, 1);
                                 }
@@ -592,7 +602,8 @@ impl RetransmitStage {
         repair_socket: Arc<UdpSocket>,
         verified_receiver: Receiver<Vec<Packets>>,
         exit: &Arc<AtomicBool>,
-        completed_slots_receivers: [CompletedSlotsReceiver; 2],
+        rpc_completed_slots_receiver: CompletedSlotsReceiver,
+        cluster_slots_update_receiver: ClusterSlotsUpdateReceiver,
         epoch_schedule: EpochSchedule,
         cfg: Option<Arc<AtomicBool>>,
         shred_version: u16,
@@ -618,8 +629,6 @@ impl RetransmitStage {
             rpc_subscriptions.clone(),
         );
 
-        let [rpc_completed_slots_receiver, cluster_completed_slots_receiver] =
-            completed_slots_receivers;
         let rpc_completed_slots_hdl =
             RpcCompletedSlotsService::spawn(rpc_completed_slots_receiver, rpc_subscriptions);
         let cluster_slots_service = ClusterSlotsService::new(
@@ -627,7 +636,7 @@ impl RetransmitStage {
             cluster_slots.clone(),
             bank_forks.clone(),
             cluster_info.clone(),
-            cluster_completed_slots_receiver,
+            cluster_slots_update_receiver,
             exit.clone(),
         );
 
@@ -709,6 +718,7 @@ mod tests {
         let (ledger_path, _blockhash) = create_new_tmp_ledger!(&genesis_config);
         let blockstore = Blockstore::open(&ledger_path).unwrap();
         let opts = ProcessOptions {
+            accounts_db_test_hash_calculation: true,
             full_leader_cache: true,
             ..ProcessOptions::default()
         };
@@ -728,8 +738,13 @@ mod tests {
             .unwrap()
             .local_addr()
             .unwrap();
-
-        let other = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
+        // This fixes the order of nodes returned by shuffle_peers_and_index,
+        // and makes turbine retransmit tree deterministic for the purpose of
+        // the test.
+        let other = std::iter::repeat_with(solana_sdk::pubkey::new_rand)
+            .find(|pk| me.id < *pk)
+            .unwrap();
+        let other = ContactInfo::new_localhost(&other, 0);
         let cluster_info = ClusterInfo::new_with_invalid_keypair(other);
         cluster_info.insert_info(me);
 
